@@ -34,11 +34,7 @@ def charFormat(color, style='', background=None):
     """
     Return a QTextCharFormat with the given attributes.
     """
-    _color = QColor()
-    if type(color) is not str:
-        _color.setRgb(color[0], color[1], color[2])
-    else:
-        _color.setNamedColor(color)
+    _color = pg.functions.mkColor(color)
 
     _format = QTextCharFormat()
     _format.setForeground(_color)
@@ -510,7 +506,9 @@ class ExampleLoader(QtWidgets.QMainWindow):
             return os.path.join(path, item.file)
         return None
 
-    def loadFile(self, edited=False):
+    def loadFile(self, *, edited=False):
+        # make *edited* keyword-only so it is not confused for extra arguments
+        # sent by ui signals
         qtLib = self.ui.qtLibCombo.currentText()
         env = dict(os.environ, PYQTGRAPH_QT_LIB=qtLib)
         example_path = os.path.abspath(os.path.dirname(__file__))
