@@ -6,10 +6,12 @@ from pyqtgraph.parametertree.parameterTypes import GroupParameter
 
 _encounteredTypes = {'group'}
 
+
 def makeChild(chType, cfgDict):
     _encounteredTypes.add(chType)
     param = Parameter.create(name='widget', type=chType)
-    param.setDefault(param.value())
+    if param.hasValue():
+        param.setDefault(param.value())
 
     def setOpt(_param, _val):
         # Treat blank strings as "None" to allow 'unsetting' that option
@@ -26,14 +28,17 @@ def makeChild(chType, cfgDict):
         else:
             optsChildren.append(child)
             child.sigValueChanged.connect(setOpt)
-    # Poplate initial options
+    # Populate initial options
     for p in optsChildren:
-        setOpt(p, p.value())
+        setOpt(p, p.value() if p.hasValue() else None)
 
-    grp = Parameter.create(name=f'Sample {chType.title()}', type='group', children=metaChildren + [param] + optsChildren)
+    grp = Parameter.create(
+        name=f'Sample {chType.title()}', type='group', children=metaChildren + [param] + optsChildren
+    )
     grp.setOpts(expanded=False)
     return grp
 
+
 def makeMetaChild(name, cfgDict):
     children = []
     for chName, chOpts in cfgDict.items():
@@ -47,6 +52,7 @@ def makeMetaChild(name, cfgDict):
     param.setOpts(expanded=False)
     return param
 
+
 def makeAllParamTypes():
     children = []
     for name, paramCfg in cfg.items():
