@@ -762,8 +762,8 @@ class ROI(GraphicsObject):
             return self.pen
 
     def contextMenuEnabled(self):
-        return self.removable
-    
+        return self.removable or self.menu and len(self.menu.children()) > 1
+
     def raiseContextMenu(self, ev):
         if not self.contextMenuEnabled():
             return
@@ -776,13 +776,11 @@ class ROI(GraphicsObject):
         if self.menu is None:
             self.menu = QtWidgets.QMenu()
             self.menu.setTitle(translate("ROI", "ROI"))
-            remAct = QtGui.QAction(translate("ROI", "Remove ROI"), self.menu)
-            remAct.triggered.connect(self.removeClicked)
-            self.menu.addAction(remAct)
-            self.menu.remAct = remAct
-        # ROI menu may be requested when showing the handle context menu, so
-        # return the menu but disable it if the ROI isn't removable
-        self.menu.setEnabled(self.contextMenuEnabled())
+            if self.removable:
+                remAct = QtGui.QAction(translate("ROI", "Remove ROI"), self.menu)
+                remAct.triggered.connect(self.removeClicked)
+                self.menu.addAction(remAct)
+                self.menu.remAct = remAct
         return self.menu
 
     def removeClicked(self):
@@ -1470,14 +1468,15 @@ class Handle(UIGraphicsItem):
         self.path = QtGui.QPainterPath()
         ang = self.startAng
         dt = 2 * np.pi / self.sides
-        for i in range(0, self.sides+1):
+        for i in range(0, self.sides):
             x = size * cos(ang)
             y = size * sin(ang)
             ang += dt
             if i == 0:
                 self.path.moveTo(x, y)
             else:
-                self.path.lineTo(x, y)            
+                self.path.lineTo(x, y)
+        self.path.closeSubpath()
             
     def paint(self, p, opt, widget):
         p.setRenderHints(p.RenderHint.Antialiasing, True)
@@ -1542,7 +1541,13 @@ class MouseDragHandler(object):
         if ev.isStart():
             if ev.button() == QtCore.Qt.MouseButton.LeftButton:
                 roi.setSelected(True)
-                mods = ev.modifiers() & ~self.snapModifier
+                mods = ev.modifiers()
+                try:
+                    mods &= ~self.snapModifier
+                except ValueError:
+                    # workaround bug in Python 3.11.4 that affects PyQt
+                    if mods & self.snapModifier:
+                        mods ^= self.snapModifier
                 if roi.translatable and mods == self.translateModifier:
                     self.dragMode = 'translate'
                 elif roi.rotatable and mods == self.rotateModifier:
