@@ -9,7 +9,7 @@ from .. import Qt, debug
 from .. import functions as fn
 from .. import getConfigOption
 from ..Point import Point
-from ..Qt import QT_LIB, QtCore, QtGui
+from ..Qt import QtCore, QtGui
 from .GraphicsObject import GraphicsObject
 
 __all__ = ['ScatterPlotItem', 'SpotItem']
@@ -85,7 +85,7 @@ def drawSymbol(painter, symbol, size, pen, brush):
     painter.drawPath(symbol)
 
 
-def renderSymbol(symbol, size, pen, brush, device=None):
+def renderSymbol(symbol, size, pen, brush, device=None, dpr=1.0):
     """
     Render a symbol specification to QImage.
     Symbol may be either a QPainterPath or one of the keys in the Symbols dict.
@@ -96,13 +96,14 @@ def renderSymbol(symbol, size, pen, brush, device=None):
     ## Render a spot with the given parameters to a pixmap
     penPxWidth = max(math.ceil(pen.widthF()), 1)
     if device is None:
-        device = QtGui.QImage(int(size+penPxWidth), int(size+penPxWidth),
-            QtGui.QImage.Format.Format_ARGB32_Premultiplied)
+        side = int(math.ceil(dpr*(size+penPxWidth)))
+        device = QtGui.QImage(side, side, QtGui.QImage.Format.Format_ARGB32_Premultiplied)
+        device.setDevicePixelRatio(dpr)
         device.fill(QtCore.Qt.GlobalColor.transparent)
     p = QtGui.QPainter(device)
     try:
         p.setRenderHint(p.RenderHint.Antialiasing)
-        p.translate(device.width()*0.5, device.height()*0.5)
+        p.translate(device.width()/dpr*0.5, device.height()/dpr*0.5)
         drawSymbol(p, symbol, size, pen, brush)
     finally:
         p.end()
@@ -133,57 +134,6 @@ def _mkBrush(*args, **kwargs):
         return fn.mkBrush(*args, **kwargs)
 
 
-class PixmapFragments:
-    def __init__(self):
-        self.use_sip_array = (
-            Qt.QT_LIB.startswith('PyQt') and
-            hasattr(Qt.sip, 'array') and
-            (
-                (0x60301 <= QtCore.PYQT_VERSION) or
-                (0x50f07 <= QtCore.PYQT_VERSION < 0x60000)
-            )
-        )
-        self.alloc(0)
-
-    def alloc(self, size):
-        # The C++ native API is:
-        #   drawPixmapFragments(const PixmapFragment *fragments, int fragmentCount,
-        #                       const QPixmap &pixmap)
-        #
-        # PySide exposes this API whereas PyQt wraps it to be more Pythonic.
-        # In PyQt, a Python list of PixmapFragment instances needs to be provided.
-        # This is inefficient because:
-        # 1) constructing the Python list involves calling sip.wrapinstance multiple times.
-        #    - this is mitigated here by reusing the instance pointers
-        # 2) PyQt will anyway deconstruct the Python list and repack the PixmapFragment
-        #    instances into a contiguous array, in order to call the underlying C++ native API.
-        if self.use_sip_array:
-            self.objs = Qt.sip.array(QtGui.QPainter.PixmapFragment, size)
-            vp = Qt.sip.voidptr(self.objs, len(self.objs)*10*8)
-            self.arr = np.frombuffer(vp, dtype=np.float64).reshape((-1, 10))
-        else:
-            self.arr = np.empty((size, 10), dtype=np.float64)
-            if QT_LIB.startswith('PyQt'):
-                self.objs = list(map(Qt.sip.wrapinstance,
-                    itertools.count(self.arr.ctypes.data, self.arr.strides[0]),
-                    itertools.repeat(QtGui.QPainter.PixmapFragment, self.arr.shape[0])))
-            else:
-                self.objs = Qt.shiboken.wrapInstance(self.arr.ctypes.data, QtGui.QPainter.PixmapFragment)
-
-    def array(self, size):
-        if size != self.arr.shape[0]:
-            self.alloc(size)
-        return self.arr
-
-    def draw(self, painter, pixmap):
-        if not len(self.arr):
-            return
-        if QT_LIB.startswith('PyQt'):
-            painter.drawPixmapFragments(self.objs, pixmap)
-        else:
-            painter.drawPixmapFragments(self.objs, len(self.arr), pixmap)
-
-
 class SymbolAtlas(object):
     """
     Used to efficiently construct a single QPixmap containing all rendered symbols
@@ -199,14 +149,8 @@ class SymbolAtlas(object):
     _idGenerator = itertools.count()
 
     def __init__(self):
-        self._data = np.zeros((0, 0, 4), dtype=np.ubyte)  # numpy array of atlas image
-        self._coords = {}
-        self._pixmap = None
-        self._maxWidth = 0
-        self._totalWidth = 0
-        self._totalArea = 0
-        self._pos = (0, 0)
-        self._rowShape = (0, 0)
+        self._dpr = 1.0
+        self.clear()
 
     def __getitem__(self, styles):
         """
@@ -224,6 +168,12 @@ class SymbolAtlas(object):
     def __len__(self):
         return len(self._coords)
 
+    def devicePixelRatio(self):
+        return self._dpr
+
+    def setDevicePixelRatio(self, dpr):
+        self._dpr = dpr
+
     @property
     def pixmap(self):
         if self._pixmap is None:
@@ -232,7 +182,8 @@ class SymbolAtlas(object):
 
     @property
     def maxWidth(self):
-        return self._maxWidth
+        # return the max logical width
+        return self._maxWidth / self._dpr
 
     def rebuild(self, styles=None):
         profiler = debug.Profiler()  # noqa: profiler prints on GC
@@ -247,7 +198,14 @@ class SymbolAtlas(object):
             self._extendFromData(data)
 
     def clear(self):
-        self.__init__()
+        self._data = np.zeros((0, 0, 4), dtype=np.ubyte)  # numpy array of atlas image
+        self._coords = {}
+        self._pixmap = None
+        self._maxWidth = 0
+        self._totalWidth = 0
+        self._totalArea = 0
+        self._pos = (0, 0)
+        self._rowShape = (0, 0)
 
     def diagnostics(self):
         n = len(self)
@@ -284,7 +242,7 @@ class SymbolAtlas(object):
         images = []
         data = []
         for key, style in styles.items():
-            img = renderSymbol(*style)
+            img = renderSymbol(*style, dpr=self._dpr)
             arr = fn.ndarray_from_qimage(img)
             images.append(img)  # keep these to delay garbage collection
             data.append((key, arr))
@@ -427,7 +385,7 @@ class ScatterPlotItem(GraphicsObject):
         self.bounds = [None, None]  ## caches data bounds
         self._maxSpotWidth = 0      ## maximum size of the scale-variant portion of all spots
         self._maxSpotPxWidth = 0    ## maximum size of the scale-invariant portion of all spots
-        self._pixmapFragments = PixmapFragments()
+        self._pixmapFragments = Qt.internals.PrimitiveArray(QtGui.QPainter.PixmapFragment, 10)
         self.opts = {
             'pxMode': True,
             'useCache': True,  ## If useCache is False, symbols are re-drawn on every paint.
@@ -970,7 +928,7 @@ class ScatterPlotItem(GraphicsObject):
         self.invalidate()
 
     @debug.warnOnException  ## raising an exception here causes crash
-    def paint(self, p, *args):
+    def paint(self, p, option, widget):
         profiler = debug.Profiler()
         cmode = self.opts.get('compositionMode', None)
         if cmode is not None:
@@ -998,17 +956,28 @@ class ScatterPlotItem(GraphicsObject):
             if self.opts['useCache'] and self._exportOpts is False:
                 # Draw symbols from pre-rendered atlas
 
+                dpr = self.fragmentAtlas.devicePixelRatio()
+                if widget is not None and (dpr_new := widget.devicePixelRatioF()) != dpr:
+                    # force a re-render if dpr changed
+                    dpr = dpr_new
+                    self.fragmentAtlas.setDevicePixelRatio(dpr)
+                    self.fragmentAtlas.clear()
+                    self.data['sourceRect'] = 0
+                    self.updateSpots()
+
                 # x, y is the center of the target rect
                 xy = pts[:, viewMask].T
                 sr = self.data['sourceRect'][viewMask]
 
-                frags = self._pixmapFragments.array(sr.size)
+                self._pixmapFragments.resize(sr.size)
+                frags = self._pixmapFragments.ndarray()
                 frags[:, 0:2] = xy
                 frags[:, 2:6] = np.frombuffer(sr, dtype=int).reshape((-1, 4)) # sx, sy, sw, sh
-                frags[:, 6:10] = [1.0, 1.0, 0.0, 1.0]   # scaleX, scaleY, rotation, opacity
+                frags[:, 6:10] = [1/dpr, 1/dpr, 0.0, 1.0]   # scaleX, scaleY, rotation, opacity
 
                 profiler('prep')
-                self._pixmapFragments.draw(p, self.fragmentAtlas.pixmap)
+                drawargs = self._pixmapFragments.drawargs()
+                p.drawPixmapFragments(*drawargs, self.fragmentAtlas.pixmap)
                 profiler('draw')
             else:
                 # render each symbol individually
