@@ -4,7 +4,10 @@ Copyright 2010-2016 Luke Campagnola
 Distributed under MIT/X11 license. See license.txt for more information.
 """
 
+import numpy
+
 from .. import functions as fn
+from .. import functions_qimage
 from .. import getConfigOption, getCupy
 from ..Qt import QtCore, QtGui, QtWidgets
 
@@ -54,10 +57,28 @@ class RawImageWidget(QtWidgets.QWidget):
         if self.opts is None:
             return
         if self.image is None:
-            argb, alpha = fn.makeARGB(self.opts[0], *self.opts[1], **self.opts[2])
-            if self._cp and self._cp.get_array_module(argb) == self._cp:
-                argb = argb.get()  # transfer GPU data back to the CPU
-            self.image = fn.makeQImage(argb, alpha, copy=False, transpose=False)
+            img = self.opts[0]
+            xp = self._cp.get_array_module(img) if self._cp else numpy
+
+            qimage = None
+            if (
+                not self.opts[1]    # no positional arguments
+                and {"levels", "lut"}.issuperset(self.opts[2])  # no kwargs besides levels and lut
+                and not (img.dtype.kind == "f" and xp.isnan(img.min()))
+            ):
+                qimage = functions_qimage.try_make_qimage(
+                    img,
+                    levels=self.opts[2].get("levels"),
+                    lut=self.opts[2].get("lut"),
+                )
+
+            if qimage is None:
+                argb, alpha = fn.makeARGB(self.opts[0], *self.opts[1], **self.opts[2])
+                if self._cp and self._cp.get_array_module(argb) == self._cp:
+                    argb = argb.get()  # transfer GPU data back to the CPU
+                qimage = fn.ndarray_to_qimage(argb, QtGui.QImage.Format.Format_ARGB32)
+
+            self.image = qimage
             self.opts = ()
         # if self.pixmap is None:
             # self.pixmap = QtGui.QPixmap.fromImage(self.image)
