@@ -11,13 +11,15 @@ app = pg.mkQApp()
 def test_ImageExporter_filename_dialog():
     """Tests ImageExporter code path that opens a file dialog. Regression test
     for pull request 1133."""
-    p = pg.plot()
+    p = pg.PlotWidget()
+    p.show()
     exp = ImageExporter(p.getPlotItem())
     exp.export()
 
 
 def test_ImageExporter_toBytes():
-    p = pg.plot()
+    p = pg.PlotWidget()
+    p.show()
     p.hideAxis('bottom')
     p.hideAxis('left')
     exp = ImageExporter(p.getPlotItem())
